"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@hibas123/utils");
const idb_1 = require("idb");
class IDB {
    constructor(database, stores, upgrade_fkt) {
        this.stores = stores;
        this.initLock = new utils_1.Lock();
        this.initLock.getLock().then(async (l) => {
            let v = localStorage.getItem(database + "_version");
            let version = 0;
            if (v)
                version = Number(v);
            let lastStoresS = localStorage.getItem(database + "_stores");
            if (!lastStoresS)
                lastStoresS = "";
            let lastStores = lastStoresS.split(",").filter(e => e !== "");
            if (!stores.every(e => lastStores.indexOf(e) >= 0) || !lastStores.every(e => stores.indexOf(e) >= 0))
                version++;
            localStorage.setItem(database + "_version", version.toString());
            localStorage.setItem(database + "_stores", stores.join(","));
            this.db = await (0, idb_1.openDB)(database, version, {
                upgrade: (db) => {
                    console.log("IndexedDB need update");
                    const check = name => db.objectStoreNames.contains ? db.objectStoreNames.contains(name) : db.objectStoreNames.find(e => e === name);
                    stores.forEach(store => {
                        if (!check(store))
                            db.createObjectStore(store);
                    });
                },
            });
            console.log("Got DATABASE", this.db);
            l.release();
            if (upgrade_fkt)
                upgrade_fkt(this.db.transaction(stores, "readwrite"));
        });
    }
    async transaction(...stores) {
        if (stores.length < 1)
            stores = this.stores;
        let s;
        if (typeof stores[0] === "string")
            s = stores;
        else
            s = stores.map(e => e.name);
        (await this.initLock.getLock()).release();
        return this.db.transaction(s, "readwrite");
    }
    getStore(name) {
        return {
            name: name,
            transaction: async () => {
                (await this.initLock.getLock()).release();
                return this.db.transaction(name, "readwrite");
            },
            get: async (key, transaction) => {
                (await this.initLock.getLock()).release();
                return (transaction || this.db.transaction(name))
                    .objectStore(name).get(key);
            },
            getAll: async (transaction) => {
                (await this.initLock.getLock()).release();
                return (transaction || this.db.transaction(name))
                    .objectStore(name).getAll();
            },
            set: async (key, val, transaction) => {
                (await this.initLock.getLock()).release();
                const tx = (transaction || this.db.transaction(name, "readwrite"));
                return tx.objectStore(name).put(val, key);
                // return tx.done;
            },
            delete: async (key, transaction) => {
                (await this.initLock.getLock()).release();
                const tx = (transaction || this.db.transaction(name, "readwrite"));
                return tx.objectStore(name).delete(key);
                // return tx.done;
            },
            clear: async (transaction) => {
                (await this.initLock.getLock()).release();
                const tx = (transaction || this.db.transaction(name, "readwrite"));
                return tx.objectStore(name).clear();
                // return tx.done;
            },
            keys: async (transaction) => {
                (await this.initLock.getLock()).release();
                const tx = (transaction || this.db.transaction(name));
                const store = tx.objectStore(name);
                return store.getAllKeys().then(res => res.map(e => e.toString()));
            }
        };
    }
}
exports.default = IDB;
//# sourceMappingURL=index.js.map